import glob
import numpy as np
import argparse 
# from importlib.util import spec_from_file_location
import os
from tqdm import tqdm
from skimage.io import imread,imsave

from MiSiC.utils import *
from MiSiC.MiSiC import *

import warnings



def main():
    warnings.filterwarnings("ignore")
    # parse arguments
    parser = argparse.ArgumentParser(description='Process images from source folder and save in destination folder')
    parser.add_argument('-lb','--light_background', help='Path of a python file containing four functions: image_io.py, pre_processing.py, processing.py,post_processing.py', required=False)
    parser.add_argument('-mw','--mean_width', help='Path of source folder with images', required=False)
    parser.add_argument('-nv','--noise_var', help='Path of source folder with images', required=False)    
    parser.add_argument('-s','--src', help='Path of source folder with images', required=True)
    parser.add_argument('-d','--dst', help='Path of destination folder', required=True)
    args = vars(parser.parse_args())

    if args['light_background'] == None:
        light_background = 'True'
        invert = True
    else:
        light_background = args['light_background']
        if light_background == 'True':
            invert = True
        else:
            invert = False

    if args['mean_width'] == None:
        mean_width = 10
    else:
        try:
            mean_width = float(args['mean_width'])
        except:
            print('Mean width not understood.')
    
    if args['src'] == None:
        print('No source images mentioned: example /path/to/src/folder/*.tif')
    else:
        src_folder = args['src']

    if args['dst'] == None:
        print('No destination directory mentioned: example /path/to/destination/*.tif')
    else:
        dst_folder = os.path.normpath(args['dst']) + os.path.sep
        # tmp = dst_folder.split('*')
        # save_format = tmp[1]
        # dst_folder = tmp[0]
    
    ## load model
    print('loading model ... ')
    misic = MiSiC()
    print('model ready.')    

    ## batch processing    
    flist = glob.glob(src_folder)
    for f in tqdm(flist):
        im = imread(f)        
        if len(im.shape)>2:
            im = im[:,:,0]
        sr,sc = im.shape
        scale = round(10.0/mean_width, 2)
        im1 = rescale(im,scale,preserve_range = False)
        im1 = add_noise(im1,sensitivity = 0.1,invert = True)
        yp = misic.segment(im1,invert=invert)
        yp = resize(yp,[sr,sc,-1])
        
        yp = postprocess_ws(img,yp)
        imsave(dst_folder + os.path.basename(f)[:-4] + '_misic.tif',yp)
    



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="MiSiC", # Replace with your own username
    version="1.0.8",
    author="S.Panigrahi",
    author_email="spanigrahi@imm.cnrs.fr",
    description="Microbe segmentation in dense colonies",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://imm.cnrs.fr",
    packages=setuptools.find_packages(),
    package_data={'': ['MiSiDC04082020.h5']},
    include_package_data=True,
    install_requires=[
   'h5py==2.10.0',
   'scikit-image',
   'tensorflow==2.1.0',
   'tqdm'
    ],
    entry_points = {
        'console_scripts': ['mbnet=MiSiC.misic_main:main'],
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.5',
)



from tensorflow.keras.models import load_model
from tensorflow.keras.utils import get_file 
import os
import matplotlib.pyplot as plt
import numpy as np
from skimage.transform import resize,rescale
from skimage.util import random_noise
from skimage.io import imread,imsave
from skimage.filters import gaussian
from skimage.feature import shape_index
from MiSiC.utils import *

########## Suggested Pre and post processing functions here   #########
from skimage.segmentation import watershed
from scipy.ndimage import label
from scipy.ndimage import gaussian_laplace
from skimage.filters import gaussian,laplace
from skimage.exposure import adjust_gamma
from skimage.color import label2rgb


def postprocess_ws(im,yp):
    '''Watershed based postprocessing using image and its pixel probabilities'''
    # mask dilated
    mask = (yp[:,:,0] > 0.4)
    # watershed potential
    d = shape_index(im,1.5)        
    
    # markers
    # get poles from contour predictions as markers
    sh = shape_index(yp[:,:,1],1)
    markers,c = label(yp[:,:,0] > 0.95)
    # ther markers should be unique to each cell 
    markers = markers*(sh<-0.5)  # only poles    
    
    ws = watershed(d, markers=markers,watershed_line = True,mask = mask,compactness = 1,connectivity = 1)        
    return ws   

def add_noise(im,sensitivity = 0.1,invert = False,seed = 42):
    '''preprocessing by adding random noise to image only where the shape index map is around -0.4 for phase contrast images'''
    t0,s = -0.4,0.025        
    sim = -shape_index(im,1)
    if invert:
        sim = -sim
    ed = np.logical_and(sim > t0-s ,sim > t0+s)*1.0
    #ed = erosion(ed)
    np.random.seed(seed)
    noise = np.random.rand(ed.shape[0],ed.shape[1])
    noise = noise*(ed+0.5*np.random.rand(ed.shape[0],ed.shape[1]) )
    img = normalize2max(im) + sensitivity*noise
    return img


class MISIC():
    def __init__(self):        
        try:
            model_path = get_file('misic_model','https://github.com/pswapnesh/Models/raw/master/MiSiDC04082020.h5') ## 0721
            self.model = load_model(model_path,compile=False)            
        except:
            model_name = os.path.join(os.path.dirname(__file__), 'MiSiDC04082020.h5');
            self.model = load_model(model_name,compile=False)

        self.model.compile(optimizer='adam',loss='categorical_crossentropy',metrics=['accuracy'])
        self.size = 256        
        
    def shapeindex_preprocess(self,im):
        ''' apply shap index map at three scales'''
        sh = np.zeros((im.shape[0],im.shape[1],3))
        if np.max(im) ==0:
            return sh
        
        # pad to minimize edge artifacts
        pw = 8
        im = np.pad(im,pw,'reflect')            
        sh[:,:,0] = shape_index(im,1)[pw:-pw,pw:-pw]
        sh[:,:,1] = shape_index(im,1.5)[pw:-pw,pw:-pw]
        sh[:,:,2] = shape_index(im,2)[pw:-pw,pw:-pw]
        #sh = 0.5*(sh+1.0)
        return sh
    
    def segment(self,im,invert = False,exclude = 16):        
        sh = self.shapeindex_preprocess(im)        
        sh = -sh if invert else sh
        tiles,params = extract_tiles(sh,size = self.size,exclude = exclude)               
        yp = self.model.predict(tiles)
        return stitch_tiles(yp,params)

    def segment_auto(self,im,invert = False):
        im1 = add_noise(im,sensitivity = 0.1,invert = invert,seed = 42)
        y = self.segment(im1,invert = invert)
        return postprocess_ws(im,y)